from .view.manager import manage as view_tc_manage
from .add.manager import manage as add_tc_manage
from .edit.manager import manage as edit_tc_manage
from .delete.manager import manage as delete_tc_manage
