from .compile.manager import manage as compile_manage
from .config.manager import manage as config_manage
from .create.manager import manage as create_manage
from .fetch.manager import manage as fetch_manage
from .run.manager import manage as run_manage
from .test.manager import manage as test_manage
from .tc import add_tc_manage, view_tc_manage, edit_tc_manage, delete_tc_manage
